@@ -156,6 +156,8 @@ struct _ClockData {
         GtkWidget *custom_entry;
         gboolean   custom_format_shown;
 
+	gboolean   can_handle_format_12;
+
 	guint listeners [N_GCONF_PREFS];
 };
 
@@ -1720,11 +1722,22 @@ applet_change_orient (PanelApplet       *applet,
 
 /* this is when the panel size changes */
 static void
-applet_change_pixel_size (PanelApplet *applet,
-			  gint         size,
-			  ClockData   *cd)
+applet_change_pixel_size (PanelApplet   *applet,
+			  GtkAllocation *allocation,
+			  ClockData     *cd)
 {
-	cd->size = size;
+	int new_size;
+
+        if (cd->orient == PANEL_APPLET_ORIENT_LEFT ||
+	    cd->orient == PANEL_APPLET_ORIENT_RIGHT)
+		new_size = allocation->width;
+	else
+		new_size = allocation->height;
+
+	if (cd->size == new_size)
+		return;
+
+	cd->size = new_size;
 
         unfix_size (cd);
 	update_timeformat (cd);
@@ -1930,8 +1943,13 @@ format_changed (GConfClient  *client,
 		return;
 
 	value = gconf_value_get_string (entry->value);
-	if (!gconf_string_to_enum (format_type_enum_map, value, &new_format) ||
-	    new_format == clock->format)
+	if (!gconf_string_to_enum (format_type_enum_map, value, &new_format))
+		return;
+
+	if (!clock->can_handle_format_12 && new_format == CLOCK_FORMAT_12)
+		new_format = CLOCK_FORMAT_24;
+
+	if (new_format == clock->format)
 		return;
 
 	clock->format = new_format;
@@ -2237,7 +2255,7 @@ fill_clock_applet (PanelApplet *applet)
 	cd->showdate = panel_applet_gconf_get_bool (applet, KEY_SHOW_DATE, &error);
 	if (error) {
 		g_error_free (error);
-		/* if on a small screen don't show data by default */
+		/* if on a small screen don't show date by default */
 		if (gdk_screen_width () <= 800)
 			cd->showdate = FALSE;
 		else
@@ -2250,6 +2268,10 @@ fill_clock_applet (PanelApplet *applet)
 
 	cd->timeformat = NULL;
 
+	cd->can_handle_format_12 = (clock_locale_format () == CLOCK_FORMAT_12);
+	if (!cd->can_handle_format_12 && cd->format == CLOCK_FORMAT_12)
+		cd->format = CLOCK_FORMAT_24;
+
 	create_clock_widget (cd);
 
 	gtk_container_set_border_width (GTK_CONTAINER (cd->applet), 0);
@@ -2268,9 +2290,8 @@ fill_clock_applet (PanelApplet *applet)
 			  G_CALLBACK (applet_change_orient),
 			  cd);
 
-	/* similiar to the above in semantics*/
 	g_signal_connect (G_OBJECT (cd->applet),
-			  "change_size",
+			  "size-allocate",
 			  G_CALLBACK (applet_change_pixel_size),
 			  cd);
 
@@ -2413,7 +2434,10 @@ set_format_cb (GtkComboBox *combo,
         ClockFormat format;
 
 	/* valid values begin from 1 */
-        format = gtk_combo_box_get_active (combo) + 1;
+	if (cd->can_handle_format_12)
+		format = gtk_combo_box_get_active (combo) + 1;
+	else
+		format = gtk_combo_box_get_active (combo) + 2;
 
         update_properties_for_format (cd, combo, format);
 
@@ -2554,7 +2578,8 @@ display_properties_dialog (BonoboUIComponent *uic,
         combo = gtk_combo_box_new_text ();
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), combo);
 
-        gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("12 hour"));
+	if (cd->can_handle_format_12)
+		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("12 hour"));
         gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("24 hour"));
         gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("UNIX time"));
         gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Internet time"));
@@ -2621,7 +2646,13 @@ display_properties_dialog (BonoboUIComponent *uic,
         update_properties_for_format (cd, GTK_COMBO_BOX (combo), cd->format);
 
 	/* valid values begin from 1 */
-	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), cd->format - 1);
+	if (cd->can_handle_format_12)
+		gtk_combo_box_set_active (GTK_COMBO_BOX (combo),
+					  cd->format - 1);
+	else
+		gtk_combo_box_set_active (GTK_COMBO_BOX (combo),
+					  cd->format - 2);
+
         g_signal_connect (combo, "changed",
                           G_CALLBACK (set_format_cb), cd);
 
