@@ -46,6 +46,7 @@
 #include "panel-a11y.h"
 #include "panel-lockdown.h"
 #include "panel-logout.h"
+#include "panel-compatibility.h"
 
 #define PANEL_ACTION_BUTTON_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_ACTION_BUTTON, PanelActionButtonPrivate))
 
@@ -72,10 +73,11 @@ static GConfEnumStringPair panel_action_type_map [] = {
 	{ PANEL_ACTION_LOGOUT,         "logout"         },
 	{ PANEL_ACTION_RUN,            "run"            },
 	{ PANEL_ACTION_SEARCH,         "search"         },
-	{ PANEL_ACTION_SCREENSHOT,     "screenshot"     },
 	{ PANEL_ACTION_FORCE_QUIT,     "force-quit"     },
 	{ PANEL_ACTION_CONNECT_SERVER, "connect-server" },
 	{ PANEL_ACTION_SHUTDOWN,       "shutdown"       },
+	/* compatibility with GNOME < 2.13.90 */
+	{ PANEL_ACTION_SCREENSHOT,     "screenshot"     },
 	{ 0,                           NULL             },
 };
 
@@ -150,13 +152,17 @@ panel_action_lock_invoke_menu (PanelActionButton *button,
 static void
 panel_action_logout (GtkWidget *widget)
 {
-	panel_logout_new (PANEL_LOGOUT_DIALOG_LOGOUT);
+	panel_logout_new (PANEL_LOGOUT_DIALOG_LOGOUT,
+			  gtk_widget_get_screen (widget),
+			  gtk_get_current_event_time ());
 }
 
 static void
 panel_action_shutdown (GtkWidget *widget)
 {
-	panel_logout_new (PANEL_LOGOUT_DIALOG_SHUTDOWN);
+	panel_logout_new (PANEL_LOGOUT_DIALOG_SHUTDOWN,
+			  gtk_widget_get_screen (widget),
+			  gtk_get_current_event_time ());
 }
 
 /* Run Application
@@ -164,7 +170,8 @@ panel_action_shutdown (GtkWidget *widget)
 static void
 panel_action_run_program (GtkWidget *widget)
 {
-	panel_run_dialog_present (gtk_widget_get_screen (widget), gtk_get_current_event_time ());
+	panel_run_dialog_present (gtk_widget_get_screen (widget),
+				  gtk_get_current_event_time ());
 }
 
 /* Search For Files
@@ -213,30 +220,6 @@ panel_action_search (GtkWidget *widget)
 	}
 }
 
-/* Take Screenshot
- */
-static void
-panel_action_screenshot (GtkWidget *widget)
-{
-	GdkScreen *screen;
-	GError    *error = NULL;
-	char      *argv [2] = {"gnome-screenshot", NULL};
-
-	screen = gtk_widget_get_screen (widget);
-
-	if (!gdk_spawn_on_screen (screen, NULL, argv, NULL,
-				  G_SPAWN_SEARCH_PATH,
-				  NULL, NULL, NULL, &error)) {
-		panel_error_dialog (screen,
-				    "cannot_exec_gnome-screenshot", TRUE,
-				    _("Could not execute '%s'"),
-				    "%s",
-				    "gnome-screenshot",
-				    error->message);
-		g_error_free (error);
-	}
-}
-
 /* Force Quit
  */
 static void
@@ -309,6 +292,9 @@ static PanelAction actions [] = {
 	{
 		PANEL_ACTION_LOGOUT,
 		"gnome-logout",
+		/* when changing one of those two strings, don't forget to
+		 * update the ones in panel-menu-items.c (look for
+		 * panel:showusername|1) */
 		N_("Log Out"),
 		N_("Log out of this session to log in as a different user or to shut down the computer"),
 		"gospanel-20",
@@ -335,15 +321,6 @@ static PanelAction actions [] = {
 		"ACTION:search:NEW",
 		panel_action_search, NULL, NULL, NULL
 	},
-	{
-		PANEL_ACTION_SCREENSHOT,
-		"applets-screenshooter",
-		N_("Take Screenshot..."),
-		N_("Take a screenshot of your desktop"),
-		"gospanel-553",
-		"ACTION:screenshot:NEW",
-		panel_action_screenshot, NULL, NULL, NULL
-	},
 	{
 		PANEL_ACTION_FORCE_QUIT,
 		PANEL_FORCE_QUIT_ICON,
@@ -372,14 +349,31 @@ static PanelAction actions [] = {
 		"ACTION:shutdown:NEW",
 		panel_action_shutdown, NULL, NULL,
 		panel_lockdown_get_disable_log_out
+	},
+	/* deprecated actions */
+	{
+		PANEL_ACTION_SCREENSHOT,
+		NULL, NULL, NULL, NULL, NULL,
+		NULL, NULL, NULL, NULL
 	}
 };
 
+static gboolean
+panel_action_get_is_deprecated (PanelActionButtonType type)
+{
+	g_return_val_if_fail (type > PANEL_ACTION_NONE && type < PANEL_ACTION_LAST, FALSE);
+
+	return (type >= PANEL_ACTION_SCREENSHOT);
+}
+
 gboolean
 panel_action_get_is_disabled (PanelActionButtonType type)
 {
 	g_return_val_if_fail (type > PANEL_ACTION_NONE && type < PANEL_ACTION_LAST, FALSE);
 
+	if (panel_action_get_is_deprecated (type))
+		return TRUE;
+
 	if (actions [type].is_disabled)
 		return actions [type].is_disabled ();
 
@@ -640,6 +634,9 @@ panel_action_button_set_type (PanelActionButton     *button,
 {
 	g_return_if_fail (type > PANEL_ACTION_NONE && type < PANEL_ACTION_LAST);
 
+	if (panel_action_get_is_deprecated (type))
+		return;
+
 	if (type == button->priv->type)
 		return;
 
@@ -820,7 +817,13 @@ panel_action_button_load_from_gconf (PanelWidget *panel,
 
 	g_free (action_type);
 
-	panel_action_button_load (type, panel, locked, position, exactpos, id, FALSE);
+	/* compatibility: migrate from GNOME < 2.13.90 */
+	if (type == PANEL_ACTION_SCREENSHOT)
+		panel_compatiblity_migrate_screenshot_action (panel_gconf_get_client (),
+							      id);
+	else
+		panel_action_button_load (type, panel, locked,
+					  position, exactpos, id, FALSE);
 }
 
 void
@@ -879,6 +882,9 @@ panel_action_button_load_from_drag (PanelToplevel *toplevel,
 
 	g_return_val_if_fail (type > PANEL_ACTION_NONE && type < PANEL_ACTION_LAST, FALSE);
 
+	if (panel_action_get_is_deprecated (type))
+		return retval;
+
 	if (strcmp (elements [2], "NEW")) {
 		*old_applet_idx = strtol (elements [2], NULL, 10);
 		retval = TRUE; /* Remove the old applet */
