@@ -157,6 +157,12 @@ panel_applet_frame_sync_menu_state (PanelAppletFrame *frame)
 				      locked_down ? "1" : "0",
 				      NULL);
 
+	bonobo_ui_component_set_prop (frame->priv->ui_component,
+				      "/commands/LockSeparator",
+				      "hidden",
+				      locked_down ? "1" : "0",
+				      NULL);
+
 	bonobo_ui_component_set_prop (frame->priv->ui_component,
 				      "/commands/RemoveAppletFromPanel",
 				      "hidden",
@@ -363,9 +369,14 @@ panel_applet_frame_load_from_gconf (PanelWidget *panel_widget,
 	g_return_if_fail (id != NULL);
 
 	key = panel_gconf_full_key (PANEL_GCONF_APPLETS, id, "bonobo_iid");
-	applet_iid = gconf_client_get_string (panel_gconf_get_client (), key, NULL);
+	applet_iid = gconf_client_get_string (panel_gconf_get_client (),
+					      key, NULL);
 
-	panel_applet_frame_load (applet_iid, panel_widget, locked, position, TRUE, id);
+	if (!applet_iid)
+		return;
+
+	panel_applet_frame_load (applet_iid, panel_widget,
+				 locked, position, TRUE, id);
 
 	g_free (applet_iid);
 }
